package Middleware;

import Database.DAO;
import SensorClass.AbstractSensorData;
import WebSocket.SocketHandler;
import com.google.gson.Gson;
import com.hivemq.client.mqtt.MqttClient;
import com.hivemq.client.mqtt.MqttGlobalPublishFilter;
import com.hivemq.client.mqtt.datatypes.MqttQos;
import com.hivemq.client.mqtt.mqtt3.Mqtt3BlockingClient;
import com.hivemq.client.mqtt.mqtt3.message.publish.Mqtt3Publish;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;

import static Middleware.XMLParser.xmlToSensorDataArrayList;
import static WebSocket.SocketServer.runSocketServer;
import static java.nio.charset.StandardCharsets.UTF_8;

import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;

/**
 * @author Pål Rebnord Brügger
 * @version 1.0
 * Java class that subscribes and writes content the database
 */
public class MiddlewareMain {
    public static void main(String[] args) throws Exception {

        String host = "05f798c5a7b1480191169623c58ff932.s2.eu.hivemq.cloud";
        String username = "SmartOceanSubscriber1";
        String password = "SmartOcean123";
        String topic1 = "Austevoll - Nord";
        String topic2 = "Austevoll";
        String topic3 = "Austevoll - Sor";

        // Create the MQTT client
        Mqtt3BlockingClient client = MqttClient.builder()
                .serverHost(host)
                .serverPort(8883)
                .sslWithDefaultConfig()
                .useMqttVersion3()
                .buildBlocking();

        // Connect to the broker
        client.connectWith()
                .simpleAuth()
                .username(username)
                .password(UTF_8.encode(password))
                .applySimpleAuth()
                .send();

        client.subscribeWith().topicFilter(topic1).qos(MqttQos.AT_LEAST_ONCE).send();
        client.subscribeWith().topicFilter(topic2).qos(MqttQos.AT_LEAST_ONCE).send();
        client.subscribeWith().topicFilter(topic3).qos(MqttQos.AT_LEAST_ONCE).send();

        System.out.println("Connected successfully to HiveMQ Cluster");


        String receivedPayload = "";

        // Starting websocket server
        runSocketServer();

        while (true) {

            try (final Mqtt3BlockingClient.Mqtt3Publishes publishes = client.publishes(MqttGlobalPublishFilter.SUBSCRIBED)) {

                Mqtt3Publish result = publishes.receive(120, TimeUnit.SECONDS).orElse(null);

                if (result != null) {
                    receivedPayload = new String(result.getPayloadAsBytes(), UTF_8);
                }
                System.out.println("Received data: " + receivedPayload.length() + "  Topic:  " + result.getTopic().toString());

                ArrayList<AbstractSensorData> sensorDataList;

                try {
                    sensorDataList = xmlToSensorDataArrayList(receivedPayload);
                } catch (ParserConfigurationException | SAXException | IOException e) {
                    sensorDataList = null;
                }

                if (!(sensorDataList == null)) {

                    Gson gson = new Gson();
                    String jsonData = gson.toJson(sensorDataList);
                    SocketHandler.sendToAll(jsonData);

                    ArrayList<AbstractSensorData> finalSensorDataList = sensorDataList;
                    Runnable dbStorageRunnable = new Runnable() {
                        @Override
                        public void run() {
                            if (result.getTopic().toString().equals("Austevoll - Nord")) {
                                DAO.StoreSensorData(finalSensorDataList, "Austevoll - Nord");
                            } else if (result.getTopic().toString().equals("Austevoll - Sor")) {
                                DAO.StoreSensorData(finalSensorDataList, "Austevoll - Sor");
                            } else {
                                DAO.StoreSensorData(finalSensorDataList, "Austevoll");
                            }
                        }
                    };
                    new Thread(dbStorageRunnable).start();
                }
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }

        }
    }
}


