package Middleware.Test;

import SensorClass.AbstractSensorData;
import org.junit.Before;
import org.junit.Test;
import org.junit.Assert;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;

import static Middleware.XMLParser.*;
public class XMLParserTest {
    String xmlTestNord;
    String xmlTestSor;
    String xmlErrorMissing;

    @Before
    public void setUp() throws Exception {
        // Initialize any necessary objects or state
        xmlTestNord = new String(Files.readAllBytes(Paths.get("src/main/java/AustevollNordData/20220510T170041_0.xml")));
        xmlTestSor = new String(Files.readAllBytes(Paths.get("src/main/java/AustevollSorData/20220526T224609.xml")));
        xmlErrorMissing = new String(Files.readAllBytes(Paths.get("src/main/java/Middleware/Test/xmlErrorTest.xml")));

    }
    @Test
    public void testXmlValueExtractorAnalogNord() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(0.3092995);
        expected.add(0.0363338);


        ArrayList<Double> actual = xmlValueExtractor(xmlTestNord, "Analog Sensors");
        Assert.assertEquals(expected, actual);
    }
    @Test
    public void testXmlValueExtractorConductivityNord() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(34.54141);
        expected.add(8.031092);
        expected.add(0.6275357);
        expected.add(388260.9);
        expected.add(107834.2);


        ArrayList<Double> actual = xmlValueExtractor(xmlTestNord, "Conductivity Sensor");
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void testXmlValueExtractorDopplerNord() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(-55.58484);
        expected.add(-55.10898);
        expected.add(-55.32204);
        expected.add(-55.91581);
        expected.add(-58.03706);
        expected.add(-57.40812);
        expected.add(-58.88258);
        expected.add(-58.19028);
        expected.add(0.0);
        expected.add(200.0);

        ArrayList<Double> actual = xmlValueExtractor(xmlTestNord, "Doppler Current Profiler Sensor");
        Assert.assertEquals(expected, actual);
    }
    @Test
    public void testXmlValueExtractorOxygenNord() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(388.2285);
        expected.add(105.0502);
        expected.add(8.040565);



        ArrayList<Double> actual = xmlValueExtractor(xmlTestNord, "Optode Sensor");
        Assert.assertEquals(expected, actual);
    }
    @Test
    public void testXmlValueExtractorSeabirdNord() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);


        ArrayList<Double> actual = xmlValueExtractor(xmlTestNord, "SeaBird SBE");
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void testXmlValueExtractorAnalogSor() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(0.817794);
        expected.add(0.0423783);


        ArrayList<Double> actual = xmlValueExtractor(xmlTestSor, "Analog Sensors");
        Assert.assertEquals(expected, actual);
    }
    @Test
    public void testXmlValueExtractorConductivitySor() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(35.2069);
        expected.add(7.46811);
        expected.add(1.56915);
        expected.add(144937.0);
        expected.add(33994.3);


        ArrayList<Double> actual = xmlValueExtractor(xmlTestSor, "Conductivity Sensor");
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void testXmlValueExtractorDopplerSor() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);
        expected.add(0.0);

        ArrayList<Double> actual = xmlValueExtractor(xmlTestSor, "Doppler Current Profiler Sensor");
        Assert.assertEquals(expected, actual);
    }
    @Test
    public void testXmlValueExtractorOxygenSor() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(305.336);
        expected.add(81.5438);
        expected.add(7.50601);

        ArrayList<Double> actual = xmlValueExtractor(xmlTestSor, "Oxygen Optode");
        Assert.assertEquals(expected, actual);
    }
    @Test
    public void testXmlValueExtractorSeabirdSor() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(10.8383);
        expected.add(3.48429);
        expected.add(20.402);
        expected.add(6.429);
        expected.add(1488.27);


        ArrayList<Double> actual = xmlValueExtractor(xmlTestSor, "SeaBird SBE");
        Assert.assertEquals(expected, actual);
    }
    @Test
    public void testXmlValueExtractorTemperatureSor() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(7.49514);

        ArrayList<Double> actual = xmlValueExtractor(xmlTestSor, "Temperature Sensor");
        Assert.assertEquals(expected, actual);
    }
    @Test
    public void testXmlValueExtractorTideSor() throws Exception {
        ArrayList<Double> expected = new ArrayList<Double>();
        expected.add(755.563);
        expected.add(7.46449);
        expected.add(755.626);
        expected.add(-20.0262);


        ArrayList<Double> actual = xmlValueExtractor(xmlTestSor, "Tide Sensor");
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void testXmlToSensorDataArrayListMissing() throws Exception {
        ArrayList<Double> expected = null;
        ArrayList<AbstractSensorData> actual = xmlToSensorDataArrayList(xmlErrorMissing);

        Assert.assertEquals(actual,expected);
    }
}
