package Middleware;

import java.io.IOException;
import java.io.StringReader;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.util.ArrayList;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import SensorClass.*;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class XMLParser {
    /**
     * @param xml          String version of an xml file
     * @param prodName prodName attribute of the Sensordata element
     * @return An ArrayList of the values within the SensorData element
     * @throws IOException
     * @throws SAXException
     * @throws ParserConfigurationException
     */

    public static ArrayList<Double> xmlValueExtractor(String xml, String prodName) throws IOException, SAXException, ParserConfigurationException {
        // Create a document builder to parse the XML string
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();

        //Create an array to store the values
        ArrayList<Double> values = new ArrayList<Double>();

        try {
            // Parse the XML string to create a Document object
            Document doc = builder.parse(new InputSource(new StringReader(xml)));
            NodeList sensorDataList = doc.getElementsByTagName("SensorData");


            //Loop through the SensorData elements
            for (int i = 0; i < sensorDataList.getLength(); i++) {

                Node sensorDataNode = sensorDataList.item(i);
                NamedNodeMap sensorDataAttributes = sensorDataNode.getAttributes();
                String pName = sensorDataAttributes.getNamedItem("ProdName").getNodeValue();

                //If the sensorDataId exists then add all the values to the values ArrayList
                if (pName.equals(prodName)) {
                    Element sensorDataElement = (Element) sensorDataNode;
                    NodeList pointList = sensorDataElement.getElementsByTagName("Point");

                    //Special case for now for the DopplerSensor because it has a thousand different vector points
                    if (pName.equals("Doppler Current Profiler Sensor")) {
                        try {
                            for (int j = 0; j < 10; j++) {
                                Node pointNode = pointList.item(j);
                                Element pointElement = (Element) pointNode;
                                String value = pointElement.getElementsByTagName("Value").item(0).getTextContent();
                                values.add(Double.parseDouble(value));
                            }
                        } catch (Exception e) {
                            values.clear();
                            values.add(0, 0.0);
                            values.add(1, 0.0);
                            values.add(2, 0.0);
                            values.add(3, 0.0);
                            values.add(4, 0.0);
                            values.add(5, 0.0);
                            values.add(6, 0.0);
                            values.add(7, 0.0);
                            values.add(8, 0.0);
                            values.add(9, 0.0);


                        }
                    } else {
                        for (int j = 0; j < pointList.getLength(); j++) {
                            Node pointNode = pointList.item(j);
                            Element pointElement = (Element) pointNode;
                            String value = pointElement.getElementsByTagName("Value").item(0).getTextContent();
                            values.add(Double.parseDouble(value));
                        }
                    }
                    break;
                }
            }
        }catch (Exception e){
            return null;
        }
        return values;
    }

    public static String extractTime(String xml) throws ParserConfigurationException, IOException, SAXException {
        // Create a document builder to parse the XML string
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();

        // Parse the XML string to create a Document object
        Document doc = builder.parse(new InputSource(new StringReader(xml)));

        NodeList timeList = doc.getElementsByTagName("Time");
        String time = timeList.item(0).getTextContent();

        return time;
    }

    public static String extractID(String xml, String prodName) throws IOException, SAXException, ParserConfigurationException {
        // Create a document builder to parse the XML string
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();

        // Parse the XML string to create a Document object
        Document doc = builder.parse(new InputSource(new StringReader(xml)));
        NodeList sensorDataList = doc.getElementsByTagName("SensorData");
        String id = null;

        for (int i = 0; i < sensorDataList.getLength(); i++) {
            Node sensorDataNode = sensorDataList.item(i);
            NamedNodeMap sensorDataAttributes = sensorDataNode.getAttributes();
            String pName = sensorDataAttributes.getNamedItem("ProdName").getNodeValue();
            if (pName.equals(prodName)) {
                id = sensorDataAttributes.getNamedItem("ID").getNodeValue();
            }
        }
        return id;
    }

    private static AnalogSensor xmlToAnalogSensor(String xml) throws ParserConfigurationException, SAXException, IOException {
        ArrayList<Double> values = xmlValueExtractor(xml, ProdNameConstants.ANALOGSENSOR);

        return new AnalogSensor(extractID(xml, ProdNameConstants.ANALOGSENSOR), LocalDateTime.now().toString(), values.get(0));
    }

    private static ConductivitySensor xmlToConductivitySensor(String xml) throws ParserConfigurationException, SAXException, IOException {
        ArrayList<Double> values = xmlValueExtractor(xml, ProdNameConstants.CONDUCTIVITYSENSOR);
        return new ConductivitySensor(extractID(xml, ProdNameConstants.CONDUCTIVITYSENSOR),LocalDateTime.now().toString(), values.get(0), values.get(1), values.get(2), values.get(3), values.get(4));
    }

    private static DopplerCurrentProfilerSensor xmlToDopplerCurrentProfilerSensor(String xml) throws ParserConfigurationException, SAXException, IOException {
        ArrayList<Double> values = xmlValueExtractor(xml, ProdNameConstants.DOPPLERCURRENTPROFILERSENSOR);
        return new DopplerCurrentProfilerSensor(extractID(xml, ProdNameConstants.DOPPLERCURRENTPROFILERSENSOR), LocalDateTime.now().toString(), values.get(0), values.get(1), values.get(2), values.get(3), values.get(4), values.get(5), values.get(6), values.get(7), values.get(8), values.get(9));
    }

    private static OxygenOptodeSensor xmlToOptodeSensor(String xml) throws ParserConfigurationException, SAXException, IOException {
        ArrayList<Double> values = xmlValueExtractor(xml, ProdNameConstants.OXYGENOPTODESENSOR);
        if (values.isEmpty()){
            values = xmlValueExtractor(xml, "Optode Sensor");
        }
        return new OxygenOptodeSensor(extractID(xml, ProdNameConstants.OXYGENOPTODESENSOR), LocalDateTime.now().toString(), values.get(0), values.get(1), values.get(2));
    }

    private static SeaBirdSensor xmlToSeaBirdSensor(String xml) throws ParserConfigurationException, SAXException, IOException {
        ArrayList<Double> values = xmlValueExtractor(xml, ProdNameConstants.SEABIRDSENSOR);
        return new SeaBirdSensor(extractID(xml, ProdNameConstants.SEABIRDSENSOR), LocalDateTime.now().toString(), values.get(0), values.get(1), values.get(2), values.get(3), values.get(4));
    }

    private static TemperatureSensor xmlToTemperatureSensor(String xml) throws ParserConfigurationException, SAXException, IOException {
        ArrayList<Double> values = xmlValueExtractor(xml, ProdNameConstants.TEMPERATURESENSOR);
        return new TemperatureSensor(extractID(xml, ProdNameConstants.TEMPERATURESENSOR), LocalDateTime.now().toString(), values.get(0));
    }

    private static TideSensor xmlToTideSensor(String xml) throws ParserConfigurationException, SAXException, IOException {
        ArrayList<Double> values = xmlValueExtractor(xml, ProdNameConstants.TIDESENSOR);
        return new TideSensor(extractID(xml, ProdNameConstants.TIDESENSOR),LocalDateTime.now().toString(), values.get(0), values.get(1), values.get(2), values.get(3));
    }

    /**
     * @param xml
     * @return an arraylist of AbstractSensorData objects in the order:
     * Analog,Conductivity,Doppler,Optode,Seabird,temperature,tide
     * @throws ParserConfigurationException
     * @throws SAXException
     * @throws IOException
     */
    public static ArrayList<AbstractSensorData> xmlToSensorDataArrayList(String xml) throws ParserConfigurationException, SAXException, IOException {
        ArrayList<AbstractSensorData> sensorDataObjects = new ArrayList<AbstractSensorData>();
        try {
            AnalogSensor analogSensor = xmlToAnalogSensor(xml);
            ConductivitySensor conductivitySensor = xmlToConductivitySensor(xml);
            DopplerCurrentProfilerSensor dopplerCurrentProfilerSensor = xmlToDopplerCurrentProfilerSensor(xml);
            OxygenOptodeSensor oxygenOptodeSensor = xmlToOptodeSensor(xml);
            SeaBirdSensor seaBirdSensor = xmlToSeaBirdSensor(xml);
            TemperatureSensor temperatureSensor = xmlToTemperatureSensor(xml);
            TideSensor tideSensor = xmlToTideSensor(xml);

            sensorDataObjects.add(analogSensor);
            sensorDataObjects.add(conductivitySensor);
            sensorDataObjects.add(dopplerCurrentProfilerSensor);
            sensorDataObjects.add(oxygenOptodeSensor);
            sensorDataObjects.add(seaBirdSensor);
            sensorDataObjects.add(temperatureSensor);
            sensorDataObjects.add(tideSensor);

        } catch (IndexOutOfBoundsException e){
            System.out.println("Error: Sensor missing in XML-file");
            return null;
        } catch (Exception e){
            System.out.println("Error: ");
            return null;
        }
        return sensorDataObjects;

    }

}
