package Simulator;

import com.hivemq.client.mqtt.MqttClient;
import com.hivemq.client.mqtt.mqtt3.Mqtt3BlockingClient;

import static java.nio.charset.StandardCharsets.UTF_8;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.Random;

/**
 * @author Pål Rebnord Brügger
 * @version 1.0
 * A simulator that publishes xml-files to a hiveMQ cluster
 */
public class HiveMQPublishSimulator {
    public static void main(String[] args) throws IOException, InterruptedException {
        String host = "05f798c5a7b1480191169623c58ff932.s2.eu.hivemq.cloud";
        String username = "SmartOceanPublisher1";
        String password = "SmartOcean123";
        String topic1 = "Austevoll - Nord";
        String topic2 = "Austevoll";
        String topic3 = "Austevoll - Sor";
        String folderPath1 = "src/main/java/AustevollNordData";
        String folderPath2 = "src/main/java/AustevollSorData";


        // Create the MQTT client/
        Mqtt3BlockingClient client = MqttClient.builder()
                .serverHost(host)
                .serverPort(8883)
                .sslWithDefaultConfig()
                .useMqttVersion3()
                .buildBlocking();

        // Connect to the broker
        client.connectWith()
                .simpleAuth()
                .username(username)
                .password(UTF_8.encode(password))
                .applySimpleAuth()
                .send();

        System.out.println("Connected successfully");

        int folderIndex = 0;

        while (true) {
            File folder1 = new File(folderPath1);
            File[] files1 = folder1.listFiles((dir, name) -> name.endsWith(".xml"));
            File selectedFile1 = files1[folderIndex % 57];
            byte[] fileBytes1 = Files.readAllBytes(selectedFile1.toPath());
            // Read the XML file as a string
            String xmlContent1 = new String(fileBytes1);

            File folder2 = new File(folderPath2);
            File[] files2 = folder2.listFiles((dir, name) -> name.endsWith(".xml"));
            File selectedFile2 = files2[folderIndex % 50];
            byte[] fileBytes2 = Files.readAllBytes(selectedFile2.toPath());
            // Read the XML file as a string
            String xmlContent2 = new String(fileBytes2);


             //Publish the XML file to a topic
            client.publishWith()
                    .topic(topic1)
                    .payload(xmlContent1.getBytes())
                    .send();

            System.out.println("Published to topic:  " + topic1);

            Thread.sleep(10000);

            client.publishWith()
                    .topic(topic2)
                    .payload(xmlContent2.getBytes())
                    .send();

            System.out.println("Published to topic:  " + topic2);

            Thread.sleep(10000);

            client.publishWith()
                    .topic(topic3)
                    .payload(selectRandomXmlFile(folderPath2).getBytes())
                    .send();

            System.out.println("Published to topic:  " + topic3);

            Thread.sleep(10000);

            folderIndex++;

        }


    }

    /**
     * @param folderPath
     * @return String containg content of a random xml-file in the given folder
     * @throws IOException
     */
    private static String selectRandomXmlFile(String folderPath) throws IOException {
        File folder = new File(folderPath);
        File[] files = folder.listFiles((dir, name) -> name.endsWith(".xml"));
        if (files == null || files.length == 0) {
            throw new IllegalArgumentException("No XML files found in directory: " + folderPath);
        }
        Random random = new Random();
        int randomIndex = random.nextInt(files.length);
        File selectedFile = files[randomIndex];
        byte[] fileBytes = Files.readAllBytes(selectedFile.toPath());
        return new String(fileBytes);
    }
}
