package WebSocket;

import Database.DAO;
import SensorClass.AbstractSensorData;
import com.google.gson.Gson;
import org.eclipse.jetty.websocket.api.Session;
import org.eclipse.jetty.websocket.api.annotations.*;

import java.util.ArrayList;
import java.util.List;

@WebSocket
public class SocketHandler {
    public static final List<Session> sessions = new ArrayList<>();
    @OnWebSocketConnect
    public void onConnect(Session session) {
        sessions.add(session);
        System.out.println("WebSocket connected: " + session.getRemoteAddress().getHostName()
                + "Number of sessions: " + sessions.size());
    }

    @OnWebSocketMessage
    public void onMessage(Session session, String message) {
        System.out.println("Received message from JS: " + message );
        // Echo the message back to the client
        try {
            ArrayList<AbstractSensorData> sensorDataList = DAO.RetrieveRecentSensorData(message);
            Gson gson = new Gson();
            String data = gson.toJson(sensorDataList);
            session.getRemote().sendString(data);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void sendToAll(String message) {
        for (Session session : sessions) {
            try {
                System.out.println("Sending message to client: " + message);
                session.getRemote().sendString(message);
            } catch (Exception e) {
                // handle exception here
                System.err.println("Error sending message to client: " + e.getMessage());
                e.printStackTrace();
            }
        }
    }

    @OnWebSocketClose
    public void onClose(Session session, int statusCode, String reason) {
        sessions.remove(session);
        System.out.println("WebSocket closed: " + statusCode + " - " + reason);
    }

    @OnWebSocketError
    public void onError(Session session, Throwable throwable) {
        System.out.println("WebSocket error: " + throwable.getMessage());
    }
}
